@@ -1,5 +1,5 @@
 #!/bin/sh
-# @(#) $Id: jpegrotate.sh 284 2016-08-29 22:23:32Z leres $ (XSE)
+# @(#) $Id: jpegrotate.sh 286 2017-01-21 18:09:55Z leres $ (XSE)
 #
 # jpegrotate - rotate jpeg images
 #
@@ -10,19 +10,20 @@
 #       jpegrotate -r file ... (rotate right)
 #       jpegrotate -f file ... (rotate 180)
 #       jpegrotate -h file ... (horizontal)
-#       jpegrotate -v file ... (vertical)
+#       jpegrotate -V file ... (vertical)
 #       jpegrotate -u file ... (undo)
 #
 
 prog="`basename $0`"
-usage="usage: ${prog} [-n] (-l|-r|-v|-h|-f|-u) file ..."
+usage="usage: ${prog} [-n] (-l|-r|-V|-h|-f|-u) file ..."
 
 dryrun=0
 option=""
 status=0
 undo=0
+automatic=1
 
-args=`getopt fhlnruv $*`
+args=`getopt fhlnruV $*`
 
 if [ $? -ne 0 ]; then
 	echo "${usage}" 1>&2
@@ -36,21 +37,25 @@ for a do
 
 	-l)
 		option="-2"
+		automatic=0
 		shift
 		;;
 
 	-r)
 		option="-9"
+		automatic=0
 		shift
 		;;
 
 	-f)
 		option="-1"
+		automatic=0
 		shift
 		;;
 
 	-h)
 		option="-F"
+		automatic=0
 		shift
 		;;
 
@@ -59,8 +64,9 @@ for a do
 		shift
 		;;
 
-	-v)
+	-V)
 		option="-f"
+		automatic=0
 		shift
 		;;
 
@@ -81,6 +87,10 @@ if [ -n "${option}" -a ${undo} -ne 0 ]; then
 	exit 1
 fi
 
+if [ ${automatic} -ne 0 ]; then
+	option="-a"
+fi
+
 if [ $# -lt 1 ]; then
 	echo "${usage}" 1>&2
 	exit 1
@@ -197,7 +207,7 @@ while [ $# -gt 0 ]; do
 	fi
 
 	# If we're autodetecting, see if anything needs to be done
-	if [ -z "${option}" ]; then
+	if [ ${automatic} -ne 0 ]; then
 		rotate=`exiftags ${f} 2>&1 |
 		    awk '
 		    /Image Orientation: Top, Left-Hand/ {
@@ -262,10 +272,6 @@ while [ $# -gt 0 ]; do
 	execute "chmod +w ${r}"
 
 	# Transform
-	if [ -z "${option}" ]; then
-		# Automatic
-		option="-a"
-	fi
 	execute "exiftran ${option} -o ${r} ${f}"
 	if [ $? -ne 0 ]; then
 		rm ${r}
