@@ -1,7 +1,7 @@
 /*
- * @(#) $Id: webindex.c 282 2016-03-06 02:22:33Z leres $ (XSE)
+ * @(#) $Id: webindex.c 287 2017-01-21 19:15:36Z leres $ (XSE)
  *
- * Copyright (c) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2013, 2015, 2016
+ * Copyright (c) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2013, 2015, 2016, 2017
  *	Craig Leres
  *
  * Redistribution and use in source and binary forms, with or without
@@ -16,7 +16,7 @@
  */
 #ifndef lint
 static const char copyright[] =
-    "@(#) Copyright (c) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2013, 2015, 2016\nCraig Leres\n";
+    "@(#) Copyright (c) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2013, 2015, 2016, 2017\nCraig Leres\n";
 #endif
 
 #include <sys/types.h>
@@ -83,8 +83,8 @@ struct webfile {
 	int w_flags;
 	int w_kbytes;			/* size in Kbytes */
 	int w_nfiles;			/* number of files if a T_DIR */
-	time_t w_time;			/* image mod/inode time */
-	time_t w_tn_time;		/* thumbnail mod time */
+	struct timespec w_tim;		/* image mod/inode timespec */
+	struct timespec w_tn_tim;	/* thumbnail mod timespec */
 	time_t w_timestamp;		/* configured picture timestamp */
 	int w_w;			/* image width */
 	int w_h;			/* image height */
@@ -262,12 +262,12 @@ process(const char *dname)
 	const char *table, *tr, *tail;
 	char *cp, *section, *prefixend;
 	int i, row, col, force, intable, savedtz;
-	time_t file_time, ind_time;
+	struct timespec file_tim, ind_tim;
 	struct webfile *wp, *webfiles;
 	struct cf *cf;
 	struct section *sp;
 	size_t nfiles, nwebfiles;
-	time_t mtime, ctime;
+	struct timespec mtim, ctim;
 	char *otz;
 	int d;
 	DIR *dirp;
@@ -289,8 +289,10 @@ process(const char *dname)
 	nfiles = 0;
 	d = -1;
 	webfiles = NULL;
-	file_time = 0;
-	ind_time = 0;
+	file_tim.tv_sec = 0;
+	file_tim.tv_nsec = 0;
+	ind_tim.tv_sec = 0;
+	ind_tim.tv_nsec = 0;
 	force = 0;
 	indexfn = defindexfn;
 	table = deftable;
@@ -374,8 +376,7 @@ process(const char *dname)
 		}
 
 		/* Get index mod time */
-		ind_time = (sbuf.st_mtime > sbuf.st_ctime) ?
-		    sbuf.st_mtime : sbuf.st_ctime;
+		ind_tim = stattim(&sbuf);
 		checkmode(indexfn, sbuf.st_mode, T_HTML);
 
 		/* If index file wasn't created by us, don't update it */
@@ -437,8 +438,8 @@ process(const char *dname)
 		/* If this is a symlink, use the newest timestamp */
 		/* XXX could do a loop here to deal with symlink chains */
 		if (S_ISLNK(sbuf.st_mode)) {
-			mtime = sbuf.st_mtime;
-			ctime = sbuf.st_mtime;
+			mtim = sbuf.st_mtim;
+			ctim = sbuf.st_mtim;
 			if (stat(dp->d_name, &sbuf) < 0) {
 				fprintf(stderr,
 				    "%s: process: stat %s%s%s: %s\n",
@@ -448,10 +449,10 @@ process(const char *dname)
 				/* XXX skip these */
 				continue;
 			}
-			if (sbuf.st_mtime < mtime)
-				sbuf.st_mtime = mtime;
-			if (sbuf.st_ctime < ctime)
-				sbuf.st_ctime = ctime;
+			if (tscmp(&sbuf.st_mtim, &mtim, <))
+				sbuf.st_mtim = mtim;
+			if (tscmp(&sbuf.st_ctim, &ctim, <))
+				sbuf.st_ctim = ctim;
 		}
 
 		/* Skip the naildir */
@@ -471,8 +472,7 @@ process(const char *dname)
 			else
 				p = "";
 			wp->w_type = str2valcase(suffix2type, p);
-			wp->w_time = (sbuf.st_mtime > sbuf.st_ctime) ?
-			    sbuf.st_mtime : sbuf.st_ctime;
+			wp->w_tim = stattim(&sbuf);
 
 			/* Mark invisible files */
 			if (invisible(wp->w_name, cf, 0))
@@ -501,8 +501,7 @@ process(const char *dname)
 						wp->w_nfiles =
 						    countfiles(wp->w_name, cf);
 				}
-				wp->w_time = (sbuf.st_mtime > sbuf.st_ctime) ?
-				    sbuf.st_mtime : sbuf.st_ctime;
+				wp->w_tim = stattim(&sbuf);
 			}
 			(void)strlcat(wp->w_name, "/", sizeof(wp->w_name));
 		}
@@ -535,8 +534,8 @@ process(const char *dname)
 			checkmode(wp->w_name, wp->w_mode, wp->w_type);
 
 		/* Keep track of oldest file */
-		if (file_time < wp->w_time)
-			file_time = wp->w_time;
+		if (tscmp(&file_tim, &wp->w_tim, <))
+			file_tim = wp->w_tim;
 
 		/* Collect file width and height and do icon bookkeeping */
 		parsethumbnail(cf, wp);
@@ -569,14 +568,14 @@ process(const char *dname)
 	 *    - newest file in directory newer than index file
 	 *
 	 */
-	if (!(force || fflag || ind_time == 0 ||
-	    cf->c_time > ind_time || file_time > ind_time)) {
+	if (!(force || fflag || ind_tim.tv_sec == 0 ||
+	    tscmp(&cf->c_tim, &ind_tim, >) || tscmp(&file_tim, &ind_tim, >))) {
 		goto bail;
 	}
 
 	if (verbose)
 		fprintf(stderr, "%s: %s %s%s%s%s\n",
-		    prog, ind_time == 0 ? "creating" : "updating",
+		    prog, ind_tim.tv_sec == 0 ? "creating" : "updating",
 		    affix, prefix, indexfn, reason);
 
 	/* Create directory if it doesn't exist */
@@ -779,7 +778,7 @@ processfile(FILE *f, struct webfile *wp, struct cf *cf,
 			/* Create/update the thumbnail image */
 			if (verbose)
 				fprintf(stderr, "%s: %s %s%s%s\n",
-				    prog, wp2->w_tn_time == 0 ?
+				    prog, wp2->w_tn_tim.tv_sec == 0 ?
 				    "creating" : "updating",
 				    affix, prefix, icon);
 			if (createthumb(wp2, icon, cf)) {
@@ -1007,7 +1006,7 @@ parsethumbnail(struct cf *cf, struct webfile *wp)
 		break;
 
 	case T_GIF:
-		(void)parsegif(wp->w_name,  &wp->w_w, &wp->w_h);
+		(void)parsegif(wp->w_name,  &wp->w_w, &wp->w_h, NULL);
 		flags |= W_HAVETN;
 		break;
 
@@ -1018,7 +1017,7 @@ parsethumbnail(struct cf *cf, struct webfile *wp)
 
 	case T_PBM:
 #ifdef HAVE_LIBPPM
-		(void)parsepbm(wp->w_name,  &wp->w_w, &wp->w_h);
+		(void)parsepbm(wp->w_name,  &wp->w_w, &wp->w_h, NULL);
 #endif
 		flags |= W_HAVETN;
 		break;
@@ -1030,18 +1029,18 @@ parsethumbnail(struct cf *cf, struct webfile *wp)
 
 	case T_TIFF:
 #ifdef HAVE_LIBTIFF
-		(void)parsetiff(wp->w_name,  &wp->w_w, &wp->w_h);
+		(void)parsetiff(wp->w_name,  &wp->w_w, &wp->w_h, NULL);
 #endif
 		flags |= W_HAVETN;
 		break;
 
 	case T_XBM:
-		(void)parsexbm(wp->w_name,  &wp->w_w, &wp->w_h);
+		(void)parsexbm(wp->w_name,  &wp->w_w, &wp->w_h, NULL);
 		flags |= W_HAVETN;
 		break;
 
 	case T_XWD:
-		(void)parsexwd(wp->w_name,  &wp->w_w, &wp->w_h);
+		(void)parsexwd(wp->w_name,  &wp->w_w, &wp->w_h, NULL);
 		flags |= W_HAVETN;
 		break;
 
@@ -1125,14 +1124,14 @@ reapthumbnails(const char *dir, struct cf *cf,
 
 			/* Get thumbnail dimensions and timestamp */
 			sprintf(temp, "%s/%s", dir, dp->d_name);
-			if (!parsejpeg(temp, &w, &h, &wp->w_tn_time)) {
+			if (!parsejpeg(temp, &w, &h, &wp->w_tn_tim)) {
 				++force;
 				reason = " (parsejpeg)";
-			} else if (wp->w_tn_time == 0) {
+			} else if (wp->w_tn_tim.tv_sec == 0) {
 				++force;
 				reason = " (no thumbnail time)";
 			} else if (wp->w_tn == NULL &&
-			    wp->w_tn_time < wp->w_time) {
+			    tscmp(&wp->w_tn_tim, &wp->w_tim, < )) {
 				/* Skip check when a configured thumbnail */
 				++force;
 				reason = " (thumbnail out of date)";
